package cn.com.qmth.examcloud.core.basic.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.bean.UserType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.StudentEntity;
import cn.com.qmth.examcloud.core.basic.service.AuthService;
import cn.com.qmth.examcloud.core.basic.service.SmsCodeService;
import cn.com.qmth.examcloud.core.basic.service.StudentService;
import cn.com.qmth.examcloud.core.basic.service.bean.LoginInfo;
import cn.com.qmth.examcloud.core.basic.service.cache.StudentCache;
import cn.com.qmth.examcloud.reports.commons.bean.OnlineStudentReport;
import cn.com.qmth.examcloud.reports.commons.bean.OnlineUserReport;
import cn.com.qmth.examcloud.reports.commons.bean.OperateReport;
import cn.com.qmth.examcloud.reports.commons.enums.OperateContent;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * {@link StatusException} 状态码范围：002XXX<br>
 * {@link ApiId}范围: 200-299<br>
 * <p>
 * 鉴权
 *
 * @author WANGWEI
 * @date 2018年5月25日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@Api(tags = "登录相关接口")
@RequestMapping("${$rmp.ctr.basic}/" + "auth")
public class AuthController extends ControllerSupport {

    @Autowired
    AuthService authService;

    @Autowired
    RedisClient redisClient;

    @Autowired
    SmsCodeService smsCodeService;

    @Autowired
    StudentRepo studentRepo;

    @Autowired
    StudentService studentService;

    @Autowired
    StudentCache studentCache;

    @Naked
    @WithoutStackTrace(true)
    @ApiOperation(value = "登入", notes = "")
    @PostMapping("login")
    public Object login(@RequestBody LoginInfo loginInfo, HttpServletRequest request) {

        Boolean alwaysOK = loginInfo.getAlwaysOK();
        if (null != alwaysOK && alwaysOK) {
            setAlwaysOKResponse();
        }

        trim(loginInfo, false);

        String realIp = request.getHeader("x-forwarded-for");
        if (StringUtils.isBlank(realIp)) {
            realIp = request.getHeader("x-real-ip");
        }
        if (StringUtils.isNotBlank(realIp)) {
            loginInfo.setClientIp(realIp);
        } else {
            loginInfo.setClientIp(null);
        }

        User user;

        String isApp = request.getHeader("isApp");
        if (StringUtils.isNotEmpty(isApp) && "true".equals(isApp)) {
            LOGGER.info("App login ... " + loginInfo.getAccountValue());
            user = authService.login(loginInfo, false);
        } else {
            user = authService.login(loginInfo, true);
        }

        // 在线数据打点 start
        if (UserType.STUDENT.equals(user.getUserType())) {
            //操作日志
            ReportsUtil.report(new OperateReport(user.getRootOrgId(), user.getUserId(), user.getUserId(), null, UserType.STUDENT, OperateContent.STUDENT_LOGIN.getDesc()));

            // 在线学生登录打点
            ReportsUtil.report(new OnlineStudentReport(user.getRootOrgId(), user.getUserId()));
        } else if (UserType.COMMON.equals(user.getUserType())) {
            // 在线用户登录打点
            ReportsUtil.report(new OnlineUserReport(user.getRootOrgId(), user.getUserId()));
        }

        // 在线数据打点 end
        if (null != alwaysOK && alwaysOK) {
            return new StatusResponseX<User>(user);
        } else {
            return user;
        }
    }

    @WithoutStackTrace(true)
    @ApiOperation(value = "登出", notes = "")
    @PostMapping("logout")
    public void logout() {
        User user = getAccessUser();
        authService.logout(user);
    }

    @Naked
    @WithoutStackTrace(true)
    @ApiOperation(value = "获取登录用户", notes = "")
    @PostMapping("getLoginUser")
    public User getLoginUser(@RequestParam String key, @RequestParam String token) {

        if (StringUtils.isBlank(key)) {
            throw new StatusException("001009", "key is blank");
        }
        if (StringUtils.isBlank(token)) {
            throw new StatusException("001010", "token is blank");
        }

        return authService.getLoginUser(key, token);
    }

    @Naked
    @WithoutStackTrace(true)
    @ApiOperation(value = "第三方普通用户接入", notes = "")
    @PostMapping("/thirdPartyAccess")
    public User thirdPartyCommonUserAccess(HttpServletRequest request, @RequestParam Long orgId,
                                           @RequestParam String loginName, @RequestParam String appId,
                                           @RequestParam String timestamp, @RequestParam String token) {
        String realIp = request.getHeader("x-forwarded-for");
        if (StringUtils.isBlank(realIp)) {
            realIp = request.getHeader("x-real-ip");
        }
        return authService.thirdPartyCommonUserAccess(orgId, loginName, appId, timestamp, token,
                realIp);
    }

    @Naked
    @WithoutStackTrace(true)
    @ApiOperation(value = "第三方学生接入", notes = "")
    @PostMapping("/thirdPartyStudentAccess")
    public User thirdPartyStudentAccess(HttpServletRequest request, @RequestParam Long rootOrgId,
                                        @RequestParam String accountType, @RequestParam String accountValue,
                                        @RequestParam String appId, @RequestParam String timestamp,
                                        @RequestParam String token) {
        String realIp = request.getHeader("x-forwarded-for");
        if (StringUtils.isBlank(realIp)) {
            realIp = request.getHeader("x-real-ip");
        }
        return authService.thirdPartyStudentAccess(rootOrgId, accountType, accountValue, appId,
                timestamp, token, realIp);
    }

    @Naked
    @WithoutStackTrace(true)
    @ApiOperation(value = "发送验证码", notes = "")
    @PostMapping("sendVerificationCode4Student")
    public void sendVerificationCode4Student(@RequestParam(required = true) String phone,
                                             @RequestParam(required = true) Boolean bound) {

        Boolean hasBeBound = studentService.hasBeBound(phone);
        if (bound && !hasBeBound) {
            throw new StatusException("002050", "手机号未被关联");
        } else if ((!bound) && hasBeBound) {
            throw new StatusException("002050", "手机号已被关联");
        }

        smsCodeService.sendSmsCode(phone);
    }

    @Naked
    @WithoutStackTrace(true)
    @ApiOperation(value = "发送验证码", notes = "")
    @PostMapping("/sendVerificationCode")
    public void sendVerificationCode(@RequestParam String phone) {
        smsCodeService.sendSmsCode(phone);
    }

    @WithoutStackTrace(true)
    @ApiOperation(value = "校验验证码", notes = "")
    @PostMapping("/checkVerificationCode")
    public void checkVerificationCode(@RequestParam String phone,
                                      @RequestParam String verificationCode) {
        smsCodeService.checkSmsCode(phone, verificationCode);
    }

    @WithoutStackTrace(true)
    @ApiOperation(value = "学生绑定手机号", notes = "")
    @PostMapping("/bindSecurityPhone")
    public void bindStudentSecurityPhone(@RequestParam String phone,
                                         @RequestParam String verificationCode) {

        smsCodeService.checkSmsCode(phone, verificationCode);

        User accessUser = getAccessUser();
        Long studentId = accessUser.getUserId();

        StudentEntity sb = studentRepo.findBySecurityPhone(phone);
        if (null != sb) {
            if (!sb.getId().equals(studentId)) {
                throw new StatusException("001052", "手机号已被其他用户绑定,请更换手机号");
            } else {
                throw new StatusException("001053", "手机号已绑定成功,无需重复绑定");
            }
        }

        StudentEntity s = GlobalHelper.getEntity(studentRepo, studentId, StudentEntity.class);
        s.setSecurityPhone(phone);
        StudentEntity saved = studentRepo.saveAndFlush(s);

        studentCache.remove(saved.getId());
    }

}