package cn.com.qmth.examcloud.core.basic.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.CourseSpeciatlyRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.SpecialtyRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseSpeciatlyRelationEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseSpeciatlyRelationPK;
import cn.com.qmth.examcloud.core.basic.dao.entity.SpecialtyEntity;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

/**
 * @author Administrator
 * @Description 课程专业关联API
 * @Date 2017年8月8号
 */
@RestController
@Api(tags = "课程专业相关接口")
@RequestMapping("${$rmp.ctr.basic}/courseSpeciatlyRelation")
public class CourseSpeciatlyRelationController extends ControllerSupport {

    @Autowired
    CourseSpeciatlyRelationRepo courseSpeciatlyRelationRepo;

    @Autowired
    CourseRepo courseRepo;

    @Autowired
    SpecialtyRepo specialtyRepo;

    @ApiOperation(value = "新增关联课程专业")
    @PostMapping("add")
    @Transactional
    public CourseSpeciatlyRelationEntity add(@RequestParam(required = true) Long courseId,
                                             @RequestParam(required = true) Long specialtyId) {
        User accessUser = getAccessUser();

        if (null == courseId) {
            throw new StatusException("320001", "courseId is null");
        }

        if (null == specialtyId) {
            throw new StatusException("320002", "specialtyId is null");
        }

        CourseEntity courseEntity = GlobalHelper.getEntity(courseRepo, courseId,
                CourseEntity.class);
        if (null == courseEntity) {
            throw new StatusException("320001", "courseId is wrong");
        }

        this.validateRootOrgIsolation(courseEntity.getRootOrgId());

        SpecialtyEntity specialtyEntity = GlobalHelper.getEntity(specialtyRepo, specialtyId,
                SpecialtyEntity.class);
        if (null == specialtyEntity) {
            throw new StatusException("320001", "specialtyId is wrong");
        }

        this.validateRootOrgIsolation(specialtyEntity.getRootOrgId());

        CourseSpeciatlyRelationPK pk = new CourseSpeciatlyRelationPK();
        pk.setCourseId(courseId);
        pk.setSpecialtyId(specialtyId);

        CourseSpeciatlyRelationEntity one = GlobalHelper.getEntity(courseSpeciatlyRelationRepo, pk,
                CourseSpeciatlyRelationEntity.class);
        if (null != one) {
            throw new StatusException("320003", "课程专业已关联");
        }

        one = new CourseSpeciatlyRelationEntity();
        one.setCourseId(courseId);
        one.setSpecialtyId(specialtyId);
        one.setCreator(accessUser.getUserId());

        CourseSpeciatlyRelationEntity saved = courseSpeciatlyRelationRepo.save(one);

        return saved;
    }

    @ApiOperation(value = "删除关联课程专业")
    @DeleteMapping("delete")
    @Transactional
    public CourseSpeciatlyRelationEntity delete(@RequestParam(required = true) Long courseId,
                                                @RequestParam(required = true) Long specialtyId) {

        if (null == courseId) {
            throw new StatusException("320001", "courseId is null");
        }

        if (null == specialtyId) {
            throw new StatusException("320002", "specialtyId is null");
        }

        CourseEntity courseEntity = GlobalHelper.getEntity(courseRepo, courseId,
                CourseEntity.class);
        if (null == courseEntity) {
            throw new StatusException("320001", "courseId is wrong");
        }

        this.validateRootOrgIsolation(courseEntity.getRootOrgId());

        SpecialtyEntity specialtyEntity = GlobalHelper.getEntity(specialtyRepo, specialtyId,
                SpecialtyEntity.class);
        if (null == specialtyEntity) {
            throw new StatusException("320001", "specialtyId is wrong");
        }

        this.validateRootOrgIsolation(specialtyEntity.getRootOrgId());

        CourseSpeciatlyRelationPK pk = new CourseSpeciatlyRelationPK();
        pk.setCourseId(courseId);
        pk.setSpecialtyId(specialtyId);

        CourseSpeciatlyRelationEntity one = GlobalHelper.getEntity(courseSpeciatlyRelationRepo, pk,
                CourseSpeciatlyRelationEntity.class);
        if (null == one) {
            throw new StatusException("320003", "课程专业未关联");
        }

        courseSpeciatlyRelationRepo.delete(one);

        return one;
    }

}
