package cn.com.qmth.examcloud.core.basic.api.controller;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.StudentFaceDomain;
import cn.com.qmth.examcloud.core.basic.dao.StudentFaceRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.FacesetEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.StudentFaceEntity;
import cn.com.qmth.examcloud.core.basic.service.FaceService;
import cn.com.qmth.examcloud.core.basic.service.bean.StudentFaceInfo;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 人脸接口<br>
 * 废弃 by wangwei 2019年9月2日
 *
 * @author WANGWEI
 * @date 2018年9月4日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
// @RestController
@Api(tags = "学生人脸相关接口")
@RequestMapping("${$rmp.ctr.basic}/face")
public class FaceController extends ControllerSupport {

    @Autowired
    StudentFaceRepo studentFaceRepo;

    @Autowired
    StudentRepo studentRepo;

    @Autowired
    FaceService studentFaceService;

    @ApiOperation(value = "保存学生人脸数据")
    @PostMapping("saveStudentFace")
    @Transactional
    public StudentFaceEntity saveStudentFace(@RequestBody StudentFaceDomain req) {
        StudentFaceInfo info = new StudentFaceInfo();
        info.setFacesetToken(req.getFacesetToken());
        info.setFaceToken(req.getFaceToken());
        info.setOperator(req.getOperator());
        info.setPhotoName(req.getPhotoName());
        info.setRootOrgId(req.getRootOrgId());
        info.setStudentId(req.getStudentId());
        info.setFaceCount(req.getFaceCount());

        StudentFaceEntity saved = studentFaceService.saveStudentFace(info);
        return saved;
    }

    @ApiOperation(value = "获取可用的faceset集合")
    @GetMapping("getUsableFacesetList")
    public List<FacesetEntity> getUsableFacesetList() {

        List<FacesetEntity> facesetList = studentFaceService.getUsableFacesetList();

        return facesetList;
    }

    @ApiOperation(value = "获取学生人脸信息")
    @GetMapping("studentFace/{studentId}")
    public StudentFaceEntity getStudentFace(@PathVariable Long studentId) {

        if (null == studentId) {
            throw new StatusException("710001", "studentId is null");
        }

        StudentFaceEntity studentFaceEntity = GlobalHelper.getEntity(studentFaceRepo, studentId,
                StudentFaceEntity.class);

        if (null == studentFaceEntity) {
            throw new StatusException("710001", "studentFaceEntity is null");
        }

        return studentFaceEntity;
    }

}
