package cn.com.qmth.examcloud.core.basic.api.controller;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.google.common.collect.Maps;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.RegExUtils;
import org.slf4j.MDC;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * 日志服务
 *
 * @author WANGWEI
 * @date 2018年11月30日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@Api(tags = "日志相关接口")
@RequestMapping("${$rmp.ctr.basic}/log")
public class LogController extends ControllerSupport {

    private static Map<String, ExamCloudLog> loggersHolder = Maps.newConcurrentMap();

    private static Object lock = new Object();

    @ApiOperation(value = "学生客户端日志")
    @PostMapping("studentClient/{level}/{code}")
    public String logByStudentClient(@PathVariable String level,
                                     @PathVariable String code,
                                     @RequestBody String info) {
        log("STUDENT_CLIENT_LOGGER", level, code, info);
        return DateUtil.chinaNow();
    }

    /**
     * 写日志
     *
     * @param logger
     * @param level
     * @param code
     * @param info
     * @author WANGWEI
     */
    private void log(String logger, String level, String code, String info) {
        if (!code.matches("\\w{1,2}\\-\\d{6}")) {
            throw new StatusException("500002", "code must match regular expression:  \\w{1,2}\\-\\d{6}");
        }

        info = RegExUtils.replaceAll(info, "[\n\r]", "  ");
        ExamCloudLog log = getLogger(logger);
        MDC.put("CODE", code);

        if ("debug".equalsIgnoreCase(level)) {
            log.debug(info);
        } else if ("info".equalsIgnoreCase(level)) {
            log.info(info);
        } else if ("error".equalsIgnoreCase(level)) {
            log.error(info);
        } else {
            throw new StatusException("500001", "level must be [debug,info,error]");
        }
    }

    /**
     * 获取日志对象
     *
     * @param logger
     * @return
     * @author WANGWEI
     */
    private ExamCloudLog getLogger(String logger) {
        ExamCloudLog log = loggersHolder.get(logger);
        if (null == log) {
            synchronized (lock) {
                if (null == log) {
                    log = ExamCloudLogFactory.getLog(logger);
                    loggersHolder.put(logger, log);
                }
                return log;
            }
        } else {
            return log;
        }
    }

}