/*
 * Copyright (c) 2020 QMTH All Rights Reserved.
 * Created by Deason on 2020-08-12 16:19:29
 */

package cn.com.qmth.examcloud.core.basic.api.controller;

import cn.com.qmth.examcloud.core.basic.service.LoginRuleService;
import cn.com.qmth.examcloud.core.basic.service.bean.LoginRuleForm;
import cn.com.qmth.examcloud.core.basic.service.bean.LoginRuleInfo;
import cn.com.qmth.examcloud.core.basic.service.bean.LoginRuleQuery;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@Api(tags = "登录规则相关接口")
@RequestMapping("${$rmp.ctr.basic}/sys/xlogin/rule")
public class LoginRuleController extends ControllerSupport {

    @Autowired
    private LoginRuleService loginRuleService;

    @PostMapping("/top/list")
    @ApiOperation(value = "查询“顶级”登录规则列表")
    public List<LoginRuleInfo> list() {
        return loginRuleService.getLoginRuleTopList();
    }

    @PostMapping("/list")
    @ApiOperation(value = "查询登录规则列表(分页)")
    public Page<LoginRuleInfo> list(@RequestBody LoginRuleQuery req) {
        return loginRuleService.getLoginRuleList(req);
    }

    @PostMapping("/save")
    @ApiOperation(value = "保存登录规则")
    public void save(@RequestBody LoginRuleForm req) {
        loginRuleService.addOrUpdateLoginRule(req);
    }

    @PostMapping("/delete")
    @ApiOperation(value = "删除某个登录规则")
    public void delete(@RequestParam Long ruleId) {
        loginRuleService.deleteLoginRuleById(ruleId);
    }

    @Naked
    @PostMapping("/refresh")
    @ApiOperation(value = "刷新登录规则")
    public void refresh() {
        loginRuleService.refreshAllLoginRule();
    }

}
