package cn.com.qmth.examcloud.core.basic.api.controller;

import cn.com.qmth.examcloud.api.commons.enums.BooleanSelect;
import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.bean.UserDataRule;
import cn.com.qmth.examcloud.api.commons.security.bean.UserType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.poi.ExcelWriter;
import cn.com.qmth.examcloud.commons.util.ByteUtil;
import cn.com.qmth.examcloud.commons.util.SHA256;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.StudentDomain;
import cn.com.qmth.examcloud.core.basic.base.constants.BasicConsts;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentCodeRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.StudentCodeEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.StudentEntity;
import cn.com.qmth.examcloud.core.basic.service.StudentService;
import cn.com.qmth.examcloud.core.basic.service.bean.StudentInfo;
import cn.com.qmth.examcloud.core.basic.service.cache.StudentCache;
import cn.com.qmth.examcloud.reports.commons.bean.OnlineStudentReport;
import cn.com.qmth.examcloud.reports.commons.bean.OperateReport;
import cn.com.qmth.examcloud.reports.commons.enums.OperateContent;
import cn.com.qmth.examcloud.reports.commons.util.ReportsUtil;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.OrgCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.StudentCacheBean;
import cn.com.qmth.examcloud.support.filestorage.FileStorageUtil;
import cn.com.qmth.examcloud.support.helper.IdentityNumberHelper;
import cn.com.qmth.examcloud.task.api.DataSyncCloudService;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.security.DataRule;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 学生服务API Created by songyue on 17/1/14.
 */
@RestController
@Api(tags = "学生相关接口")
@RequestMapping("${$rmp.ctr.basic}/student")
public class StudentController extends ControllerSupport {

    @Autowired
    StudentRepo studentRepo;

    @Autowired
    StudentCodeRepo studentCodeRepo;

    @Autowired
    SystemProperties systemConfig;

    @Autowired
    StudentService studentService;

    @Autowired
    UserRepo userRepo;

    @Autowired
    OrgRepo orgRepo;

    @Autowired
    DataSyncCloudService dataSyncCloudService;

    @Autowired
    StudentCache studentCache;

    private static final String[] EXCEL_HEADER = new String[]{"ID", "姓名", "学号", "身份证", "学习中心代码",
            "学习中心名称"};

    /**
     * 方法注释
     *
     * @param name
     * @param studentCode
     * @param identityNumber
     * @param curPage
     * @param pageSize
     * @return
     * @author WANGWEI
     */
    @DataRule(type = DataRuleType.ORG)
    @ApiOperation(value = "查询学生分页数据", notes = "分页")
    @GetMapping("studentPage/{curPage}/{pageSize}")
    public PageInfo<StudentDomain> getStudentPage(
            @PathVariable Integer curPage,
            @PathVariable Integer pageSize, @RequestParam String name,
            @RequestParam String studentCode, @RequestParam String identityNumber,
            @RequestParam(required = false) Long rootOrgId,
            @RequestParam(required = false) Long orgId,
            @RequestParam(required = false) BooleanSelect enable,
            @RequestParam(required = false) BooleanSelect hasPhoto) {

        User accessUser = getAccessUser();
        if (null == rootOrgId) {
            rootOrgId = accessUser.getRootOrgId();
        }

        validateRootOrgIsolation(rootOrgId);
        final Long finalRootOrgId = rootOrgId;

        PageRequest pageable = PageRequest.of(curPage, pageSize,
                Sort.by(Direction.DESC, "updateTime", "id"));

        UserDataRule userDataRule = super.getUserDataRule(DataRuleType.ORG);
        if (userDataRule.assertEmptyQueryResult()) {
            return new PageInfo(new PageImpl<>(new ArrayList<>(), pageable, 0L));
        }

        Specification<StudentEntity> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.equal(root.get("rootOrgId"), finalRootOrgId));

            if (StringUtils.isNotEmpty(name)) {
                predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
            }

            if (StringUtils.isNotBlank(identityNumber)) {
                predicates.add(cb.like(root.get("identityNumber"), toSqlRightLike(identityNumber)));
            }

            if (null != enable) {
                Boolean enableBoolean = enable.getBoolean();
                if (null != enableBoolean) {
                    predicates.add(cb.equal(root.get("enable"), enableBoolean));
                }
            }

            if (null != hasPhoto) {
                Boolean hasPhotoBoolean = hasPhoto.getBoolean();
                if (null != hasPhotoBoolean) {
                    if (hasPhotoBoolean) {
                        predicates.add(cb.isNotNull(root.get("photoPath")));
                    } else {
                        predicates.add(cb.isNull(root.get("photoPath")));
                    }
                }
            }

            if (StringUtils.isNotBlank(studentCode)) {
                Subquery<StudentCodeEntity> subquery = query.subquery(StudentCodeEntity.class);
                Root<StudentCodeEntity> subRoot = subquery.from(StudentCodeEntity.class);
                subquery.select(subRoot.get("id"));
                Predicate p1 = cb.equal(subRoot.get("studentId"), root.get("id"));
                Predicate p2 = cb.like(subRoot.get("studentCode"), toSqlRightLike(studentCode));
                subquery.where(cb.and(p1, p2));
                predicates.add(cb.exists(subquery));
            }


            if (orgId == null) {
                if (userDataRule.assertNeedQueryRefIds()) {
                    predicates.add(root.get("orgId").in(userDataRule.getRefIds()));
                }
            } else {
                predicates.add(cb.equal(root.get("orgId"), orgId));
            }


            /*Boolean orgStudentData = PrivilegeManager.judge(finalRootOrgId,
                    accessUser.getRoleList(), OrgStudentData.CODE);
            Boolean allStudentData = PrivilegeManager.judge(finalRootOrgId,
                    accessUser.getRoleList(), AllStudentData.CODE);
            // 学习中心过滤
            if (isSuperAdmin()) {
                if (null != orgId) {
                    predicates.add(cb.equal(root.get("orgId"), orgId));
                }
            } else if (allStudentData) {
                if (null != orgId) {
                    predicates.add(cb.equal(root.get("orgId"), orgId));
                }
            } else {
                if (orgStudentData) {
                    if (hasRole(RoleMeta.LC_USER)) {
                        UserEntity user = GlobalHelper.getEntity(userRepo, accessUser.getUserId(),
                                UserEntity.class);
                        if (null != user.getOrgId()) {
                            predicates.add(cb.equal(root.get("orgId"), user.getOrgId()));
                        } else {
                            predicates.add(cb.equal(root.get("orgId"), -1));
                        }
                    } else {
                        predicates.add(cb.equal(root.get("orgId"), -1));
                    }
                } else {
                    predicates.add(cb.equal(root.get("orgId"), -1));
                }
            }*/

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Page<StudentEntity> studentList = studentRepo.findAll(specification, pageable);
        Iterator<StudentEntity> iterator = studentList.iterator();
        List<StudentDomain> studentDomainList = Lists.newArrayList();

        //		String upyunDomain = PropertyHolder.getString("$upyun.site.1.domain");
        //		if (StringUtils.isBlank(upyunDomain)) {
        //			throw new StatusException("560111", "property[$upyun.site.1.domain] is not configured");
        //		}

        while (iterator.hasNext()) {
            StudentEntity next = iterator.next();
            StudentDomain bean = new StudentDomain();
            bean.setId(next.getId());
            bean.setName(next.getName());
            bean.setRootOrgId(next.getRootOrgId());
            bean.setUpdateTime(next.getUpdateTime());
            bean.setCreationTime(next.getCreationTime());
            bean.setEnable(next.getEnable());
            List<StudentCodeEntity> studentCodeEntityList = studentCodeRepo
                    .findByStudentId(next.getId());
            List<String> studentCodeList = Lists.newArrayList();
            for (StudentCodeEntity cur : studentCodeEntityList) {
                studentCodeList.add(cur.getStudentCode());
            }
            bean.setStudentCodeList(studentCodeList);
            bean.setStudentCodesStr(StringUtils.join(studentCodeList, "<br />"));

            bean.setIdentityNumber(next.getIdentityNumber());
            bean.setPrivateIdentityNumber(IdentityNumberHelper.conceal(next.getRootOrgId(), next.getIdentityNumber()));
            if (StringUtils.isNotBlank(next.getPhotoPath())) {
                //				bean.setPhotoPath(
                //						UrlUtil.joinUrl(upyunDomain, "student_base_photo", next.getPhotoPath()));
                //通用存储
                bean.setPhotoPath(FileStorageUtil.realPath(FileStorageUtil.getIntactPath("student_base_photo",
                        next.getPhotoPath())));
            }
            bean.setPhoneNumber(next.getPhoneNumber());
            bean.setSecurityPhone(next.getSecurityPhone());
            bean.setRemark(next.getRemark());

            bean.setOrgId(next.getOrgId());
            if (null != bean.getOrgId()) {
                OrgEntity org = GlobalHelper.getEntity(orgRepo, Long.valueOf(bean.getOrgId()),
                        OrgEntity.class);
                if (null != org) {
                    bean.setOrgName(org.getName());
                    bean.setOrgCode(org.getCode());
                }
            }

            studentDomainList.add(bean);
        }

        PageInfo<StudentDomain> ret = new PageInfo<StudentDomain>();
        ret.setList(studentDomainList);
        ret.setTotal(studentList.getTotalElements());
        return ret;
    }

    @DataRule(type = DataRuleType.ORG)
    @ApiOperation(value = "导出学生", notes = "")
    @GetMapping("export")
    public void exportStudents(@RequestParam String name,
                               @RequestParam String studentCode,
                               @RequestParam String identityNumber,
                               @RequestParam(required = false) Long rootOrgId,
                               @RequestParam(required = false) Long orgId,
                               @RequestParam(required = false) BooleanSelect hasPhoto) {

        User accessUser = getAccessUser();
        if (null == rootOrgId) {
            rootOrgId = accessUser.getRootOrgId();
        }

        validateRootOrgIsolation(rootOrgId);
        final Long finalRootOrgId = rootOrgId;

        UserDataRule userDataRule = super.getUserDataRule(DataRuleType.ORG);
        if (userDataRule.assertEmptyQueryResult()) {
            this.doExport(new ArrayList<>());
            return;
        }

        Specification<StudentEntity> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.equal(root.get("rootOrgId"), finalRootOrgId));

            if (StringUtils.isNotEmpty(name)) {
                predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
            }

            if (StringUtils.isNotBlank(identityNumber)) {
                predicates.add(cb.like(root.get("identityNumber"), toSqlRightLike(identityNumber)));
            }

            if (null != hasPhoto) {
                Boolean hasPhotoBoolean = hasPhoto.getBoolean();
                if (null != hasPhotoBoolean) {
                    if (hasPhotoBoolean) {
                        predicates.add(cb.isNotNull(root.get("photoPath")));
                    } else {
                        predicates.add(cb.isNull(root.get("photoPath")));
                    }
                }
            }

            if (StringUtils.isNotBlank(studentCode)) {
                Subquery<StudentCodeEntity> subquery = query.subquery(StudentCodeEntity.class);
                Root<StudentCodeEntity> subRoot = subquery.from(StudentCodeEntity.class);
                subquery.select(subRoot.get("id"));
                Predicate p1 = cb.equal(subRoot.get("studentId"), root.get("id"));
                Predicate p2 = cb.like(subRoot.get("studentCode"), toSqlRightLike(studentCode));
                subquery.where(cb.and(p1, p2));
                predicates.add(cb.exists(subquery));
            }


            if (orgId == null) {
                if (userDataRule.assertNeedQueryRefIds()) {
                    predicates.add(root.get("orgId").in(userDataRule.getRefIds()));
                }
            } else {
                predicates.add(cb.equal(root.get("orgId"), orgId));
            }


            /*Boolean orgStudentData = PrivilegeManager.judge(finalRootOrgId,
                    accessUser.getRoleList(), OrgStudentData.CODE);
            Boolean allStudentData = PrivilegeManager.judge(finalRootOrgId,
                    accessUser.getRoleList(), AllStudentData.CODE);
            // 学习中心过滤
            if (isSuperAdmin()) {
                if (null != orgId) {
                    predicates.add(cb.equal(root.get("orgId"), orgId));
                }
            } else if (allStudentData) {
                if (null != orgId) {
                    predicates.add(cb.equal(root.get("orgId"), orgId));
                }
            } else {
                if (orgStudentData) {
                    if (hasRole(RoleMeta.LC_USER)) {
                        UserEntity user = GlobalHelper.getEntity(userRepo, accessUser.getUserId(),
                                UserEntity.class);
                        if (null != user.getOrgId()) {
                            predicates.add(cb.equal(root.get("orgId"), user.getOrgId()));
                        } else {
                            predicates.add(cb.equal(root.get("orgId"), -1));
                        }
                    } else {
                        predicates.add(cb.equal(root.get("orgId"), -1));
                    }
                } else {
                    predicates.add(cb.equal(root.get("orgId"), -1));
                }
            }*/

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        long count = studentRepo.count(specification);
        if (100000 < count) {
            throw new StatusException("520200", "数据量超过100000,无法导出");
        }

        List<StudentEntity> studentList = studentRepo.findAll(specification,
                Sort.by(Direction.DESC, "updateTime"));
        this.doExport(studentList);
    }

    private void doExport(List<StudentEntity> studentList) {
        List<Object[]> datas = Lists.newArrayList();

        for (StudentEntity cur : studentList) {
            String orgCode = null;
            String orgName = null;
            if (null != cur.getOrgId()) {
                OrgEntity org = GlobalHelper.getEntity(orgRepo, Long.valueOf(cur.getOrgId()),
                        OrgEntity.class);
                if (null != org) {
                    orgCode = org.getCode();
                    orgName = org.getName();
                }
            }

            List<StudentCodeEntity> studentCodeEntityList = studentCodeRepo
                    .findByStudentId(cur.getId());
            List<String> studentCodeList = Lists.newArrayList();
            for (StudentCodeEntity sc : studentCodeEntityList) {
                studentCodeList.add(sc.getStudentCode());
            }

            datas.add(
                    new Object[]{cur.getId(), cur.getName(), StringUtils.join(studentCodeList, ";"),
                            cur.getIdentityNumber(), orgCode, orgName});
        }

        String filePath = systemConfig.getTempDataDir() + File.separator
                + System.currentTimeMillis() + ".xlsx";
        File file = new File(filePath);

        ExcelWriter.write(EXCEL_HEADER, new Class[]{Long.class, String.class, String.class,
                String.class, String.class, String.class}, datas, new File(filePath));

        exportFile("学生-" + getRootOrgId() + ".xlsx", file);

        FileUtils.deleteQuietly(file);
    }

    /**
     * 方法注释
     *
     * @param ids
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "启用学生")
    @PutMapping("enable/{ids}")
    @Transactional
    public List<String> enableStudent(@PathVariable String ids) {
        List<Long> studentIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        List<String> ret = Lists.newArrayList();
        for (Long cur : studentIds) {
            StudentEntity s = GlobalHelper.getEntity(studentRepo, cur, StudentEntity.class);
            s.setEnable(true);
            studentRepo.save(s);
            //操作日志
            ReportsUtil.report(new OperateReport(s.getRootOrgId(), getAccessUser().getUserId(), s.getId(), null, UserType.COMMON,
                    OperateContent.STUDENT_ENABLE.getDesc()));
            ret.add(s.getId() + ":" + s.getName());
        }
        for (Long cur : studentIds) {
            studentCache.remove(cur);
        }
        return ret;
    }

    /**
     * 方法注释
     *
     * @param ids
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "禁用学生")
    @PutMapping("disable/{ids}")
    @Transactional
    public List<String> disableStudent(@PathVariable String ids) {
        List<Long> studentIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        List<String> ret = Lists.newArrayList();
        for (Long cur : studentIds) {
            StudentEntity s = GlobalHelper.getEntity(studentRepo, cur, StudentEntity.class);
            s.setEnable(false);
            studentRepo.save(s);
            //操作日志
            ReportsUtil.report(new OperateReport(s.getRootOrgId(), getAccessUser().getUserId(), s.getId(), null, UserType.COMMON,
                    OperateContent.STUDENT_DISABLE.getDesc()));
            ret.add(s.getId() + ":" + s.getName());
        }
        for (Long cur : studentIds) {
            studentCache.remove(cur);
        }
        return ret;
    }

    @ApiOperation(value = "重置学生密码", notes = "重置学生密码")
    @PutMapping("resetPass/{ids}")
    @Transactional
    public void resetPassword(@PathVariable String ids) {
        List<Long> studentIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        for (Long cur : studentIds) {
            StudentEntity s = GlobalHelper.getEntity(studentRepo, cur, StudentEntity.class);
            if (null == s) {
                throw new StatusException("450110", "学生不存在");
            }
            String identityNumber = s.getIdentityNumber();
            String passwd = null;
            if (StringUtils.isNotEmpty(identityNumber)
                    && identityNumber.matches("[0-9a-zA-Z]{6,}")) {
                passwd = StringUtils.substring(identityNumber, -6, identityNumber.length());
            } else {
                passwd = BasicConsts.DEFAULT_PASSWORD;
            }
            byte[] bytes = SHA256.encode(identityNumber + passwd);
            String encodePassword = ByteUtil.toHexAscii(bytes);
            s.setPassword(encodePassword);
            s.setUpdateTime(new Date());
            studentRepo.save(s);
            //操作日志
            ReportsUtil.report(new OperateReport(s.getRootOrgId(), getAccessUser().getUserId(), s.getId(), null, UserType.COMMON,
                    OperateContent.STUDENT_PASSWORD_RESET.getDesc()));
        }
    }

    @ApiOperation(value = "重置机构下所有学生密码", notes = "重置机构下所有学生密码")
    @PutMapping("resetPasswordByOrgId/{orgId}")
    @Transactional
    public void resetPasswordByOrgId(@PathVariable Long orgId) {
        OrgEntity org = GlobalHelper.getPresentEntity(orgRepo, orgId, OrgEntity.class);

        validateRootOrgIsolation(org.getRootId());

        List<StudentEntity> stuentList = studentRepo.findByOrgId(org.getId());
        for (StudentEntity s : stuentList) {
            String identityNumber = s.getIdentityNumber();
            String passwd = null;
            if (StringUtils.isNotEmpty(identityNumber)
                    && identityNumber.matches("[0-9a-zA-Z]{6,}")) {
                passwd = StringUtils.substring(identityNumber, -6, identityNumber.length());
            } else {
                passwd = BasicConsts.DEFAULT_PASSWORD;
            }
            byte[] bytes = SHA256.encode(identityNumber + passwd);
            String encodePassword = ByteUtil.toHexAscii(bytes);
            s.setPassword(encodePassword);
            s.setUpdateTime(new Date());
            studentRepo.save(s);
            //操作日志
            ReportsUtil.report(new OperateReport(s.getRootOrgId(), getAccessUser().getUserId(), s.getId(), null, UserType.COMMON,
                    OperateContent.STUDENT_PASSWORD_RESET.getDesc()));
        }
    }

    /**
     * 方法注释
     *
     * @param newPassword
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "修改学生密码", notes = "修改密码")
    @PutMapping("password/direct")
    @Transactional
    public Long updateStudentPassword(@RequestParam("newPassword") String newPassword) {
        User accessUser = getAccessUser();
        if (StringUtils.isEmpty(newPassword)) {
            throw new StatusException("450201", "新密码为空");
        }

        if (!newPassword.matches("[a-zA-Z0-9]{6,18}")) {
            throw new StatusException("450203", "密码必须是6至18位字母或数字");
        }

        StudentEntity s = GlobalHelper.getEntity(studentRepo, accessUser.getUserId(),
                StudentEntity.class);
        byte[] bytes = SHA256.encode(s.getIdentityNumber() + newPassword);
        String encodePassword = ByteUtil.toHexAscii(bytes);
        s.setPassword(encodePassword);
        studentRepo.save(s);
        //操作日志
        ReportsUtil.report(new OperateReport(s.getRootOrgId(), getAccessUser().getUserId(), s.getId(), null, UserType.STUDENT,
                OperateContent.STUDENT_PASSWORD_UPDATE.getDesc()));
        return s.getId();
    }

    /**
     * 方法注释
     *
     * @param password
     * @param newPassword
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "修改学生密码", notes = "修改密码")
    @PutMapping("password")
    @Transactional
    public Long updateStudentPassword(@RequestParam("password") String password,
                                      @RequestParam("newPassword") String newPassword) {
        User accessUser = getAccessUser();
        if (StringUtils.isEmpty(password)) {
            throw new StatusException("450110", "旧密码为空");
        }
        if (StringUtils.isEmpty(newPassword)) {
            throw new StatusException("450111", "新密码为空");
        }
        if (!newPassword.matches("[a-zA-Z0-9]{6,18}")) {
            throw new StatusException("450111", "密码必须是6至18位字母或数字");
        }

        StudentEntity s = GlobalHelper.getEntity(studentRepo, accessUser.getUserId(),
                StudentEntity.class);

        String encodeOldPassword = ByteUtil.toHexAscii(SHA256.encode(s.getIdentityNumber() + password));
        if (StringUtils.isNotBlank(s.getPassword()) && !s.getPassword().equals(encodeOldPassword)) {
            throw new StatusException("450111", "当前密码错误");
        }
        byte[] bytes = SHA256.encode(s.getIdentityNumber() + newPassword);
        String encodePassword = ByteUtil.toHexAscii(bytes);
        s.setPassword(encodePassword);
        studentRepo.save(s);
        //操作日志
        ReportsUtil.report(new OperateReport(s.getRootOrgId(), getAccessUser().getUserId(), s.getId(), null, UserType.STUDENT,
                OperateContent.STUDENT_PASSWORD_UPDATE.getDesc()));
        return s.getId();
    }

    /**
     * 查询学生
     *
     * @return
     * @author WANGWEI
     */
    @ApiOperation(value = "查询学生")
    @GetMapping("getStudentInfoBySession")
    public StudentInfo getStudentInfoBySession() {
        User accessUser = getAccessUser();
        Long rootOrgId = accessUser.getRootOrgId();
        Long studentId = accessUser.getUserId();

        StudentCacheBean s = CacheHelper.getStudent(studentId);

        OrgCacheBean org = CacheHelper.getOrg(s.getOrgId());
        OrgCacheBean rootOrg = CacheHelper.getOrg(rootOrgId);

        StudentInfo info = new StudentInfo();

        info.setEnable(s.getEnable());
        info.setId(s.getId());
        info.setIdentityNumber(s.getIdentityNumber());
        info.setName(s.getName());
        info.setOrgCode(org.getCode());
        info.setOrgId(org.getId());
        info.setOrgName(org.getName());
        info.setPhoneNumber(s.getPhoneNumber());
        info.setPhotoPath(s.getPhotoPath());
        info.setRemark(s.getRemark());
        info.setRootOrgId(rootOrgId);
        info.setRootOrgName(rootOrg.getName());
        info.setSecurityPhone(s.getSecurityPhone());
        info.setStudentCodeList(s.getStudentCodeList());

        return info;
    }

    @ApiOperation(value = "查询学生")
    @GetMapping("getStudentInfo")
    public StudentInfo getStudentInfo(@RequestParam(required = false) Long rootOrgId,
                                      @RequestParam(required = false) Long studentId,
                                      @RequestParam(required = false) String identityNumber,
                                      @RequestParam(required = false) String studentCode,
                                      @RequestParam(required = false) String securityPhone) {
        if (null == rootOrgId) {
            rootOrgId = getRootOrgId();
        } else {
            validateRootOrgIsolation(rootOrgId);
        }

        StudentInfo studentInfo = studentService.getStudentInfo(rootOrgId, studentId,
                identityNumber, studentCode, securityPhone);

        if (null != studentInfo) {
            studentInfo.setIdentityNumber(IdentityNumberHelper.conceal(rootOrgId, studentInfo.getIdentityNumber()));
        }

        return studentInfo;
    }

    @ApiOperation(value = "解绑学号", notes = "")
    @PostMapping("unbindStudentCode")
    @Transactional
    public List<Long> unbindStudentCode(@RequestParam(required = false) String studentCode,
                                        @RequestParam(required = false) String identityNumber,
                                        @RequestParam(required = true) Long rootOrgId) {

        if (null == rootOrgId) {
            rootOrgId = getRootOrgId();
        } else {
            OrgEntity rootOrg = GlobalHelper.getEntity(orgRepo, rootOrgId, OrgEntity.class);
            if (null == rootOrg) {
                throw new StatusException("450110", "rootOrgId is wrong");
            }
            validateRootOrgIsolation(rootOrgId);
        }

        List<Long> studentIdList = studentService.unbindStudentCode(rootOrgId, studentCode,
                identityNumber);

        for (Long cur : studentIdList) {
            studentCache.remove(cur);
            //操作日志
            ReportsUtil.report(new OperateReport(rootOrgId, getAccessUser().getUserId(), cur, null, UserType.COMMON,
                    OperateContent.STUDENT_UNBIND_STUDENTCODE.getDesc()));
        }

        return studentIdList;
    }

    @ApiOperation(value = "解绑安全手机", notes = "")
    @PutMapping("unbindSecurityPhone/{ids}")
    @Transactional
    public void unbindSecurityPhone(@PathVariable String ids) {
        List<Long> studentIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        for (Long cur : studentIds) {
            StudentEntity s = GlobalHelper.getEntity(studentRepo, cur, StudentEntity.class);
            if (null == s) {
                throw new StatusException("450110", "学生不存在");
            }
            validateRootOrgIsolation(s.getRootOrgId());
        }

        for (Long cur : studentIds) {
            studentService.unbindSecurityPhone(cur);
            StudentEntity s = GlobalHelper.getEntity(studentRepo, cur, StudentEntity.class);
            //操作日志
            ReportsUtil.report(new OperateReport(s.getRootOrgId(), getAccessUser().getUserId(), s.getId(), null, UserType.COMMON,
                    OperateContent.STUDENT_UNBIND_PHONE.getDesc()));
        }

        for (Long cur : studentIds) {
            studentCache.remove(cur);
        }

    }

    @ApiOperation(value = "学生在线信号", notes = "学生在线信号")
    @GetMapping("online_signal/{studentId}")
    public void onlineSignal(@PathVariable Long studentId) {
        User accessUser = getAccessUser();

        //在线学生
        ReportsUtil.report(new OnlineStudentReport(accessUser.getRootOrgId(), studentId));
    }

}
