package cn.com.qmth.examcloud.core.basic.api.controller;

import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.commons.util.PathUtil;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.SysNoticeDomain;
import cn.com.qmth.examcloud.core.basic.dao.SysNoticeRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.SysNoticeEntity;
import cn.com.qmth.examcloud.support.filestorage.FileStorageUtil;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.filestorage.FileStoragePathEnvInfo;
import cn.com.qmth.examcloud.web.filestorage.YunPathInfo;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.mysql.cj.util.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.io.IOException;

/**
 * @Description 系统通知 状态码范围007XXX
 * @Author lideyin
 * @Date 2020/8/16 10:49
 * @Version 1.0
 */
@RestController
@Api(tags = "系统通知")
@RequestMapping("${$rmp.ctr.basic}/sys/notice")
public class SysNoticeController extends ControllerSupport {

    @Autowired
    SysNoticeRepo sysNoticeRepo;
    @Autowired
    SystemProperties systemConfig;

    @ApiOperation(value = "查询系统通知")
    @GetMapping("getSysNotice")
    public SysNoticeDomain getSysNotice(@RequestParam(required = false) Long rootOrgId) {
        //不传默认取全部
        if (null == rootOrgId) {
            rootOrgId = -1L;
        }

        SysNoticeEntity entity = sysNoticeRepo.findTopByRootOrgIdOrderByIdDesc(rootOrgId);
        if (null == entity) {
            return null;
        }

        SysNoticeDomain domain = new SysNoticeDomain();
        domain.setId(entity.getId());
        domain.setRootOrgId(entity.getRootOrgId());
        domain.setContent(entity.getContent());
        domain.setStartTime(DateUtil.format(entity.getStartTime(), DateUtil.DatePatterns.CHINA_DEFAULT));
        domain.setEndTime(DateUtil.format(entity.getEndTime(), DateUtil.DatePatterns.CHINA_DEFAULT));
        domain.setTitle(entity.getTitle());
        domain.setEnable(entity.getEnable());

        return domain;
    }

    @ApiOperation(value = "保存系统通知", notes = "")
    @PutMapping("saveSysNotice")
    public String saveOrgProperties(@RequestBody SysNoticeDomain domain) {
        if (StringUtils.isNullOrEmpty(domain.getContent())) {
            throw new StatusException("007001", "通知标题不允许为空");
        }

        if (StringUtils.isNullOrEmpty(domain.getContent())) {
            throw new StatusException("007002", "通知内容不允许为空");
        }

        if (StringUtils.isNullOrEmpty(domain.getStartTime()) ||
                StringUtils.isNullOrEmpty(domain.getEndTime())) {
            throw new StatusException("007003", "通知有效期不允许为空");
        }

        if (StringUtils.isNullOrEmpty(domain.getContent())) {
            throw new StatusException("007004", "通知状态不允许为空");
        }

        SysNoticeEntity entity = null;
        //新增
        if (null == domain.getId()) {
            entity = new SysNoticeEntity();
            entity.setRootOrgId(domain.getRootOrgId() == null ? -1 : domain.getRootOrgId());
            entity.setContent(domain.getContent());
            entity.setStartTime(DateUtil.parse(domain.getStartTime(), DateUtil.DatePatterns.CHINA_DEFAULT));
            entity.setEndTime(DateUtil.parse(domain.getEndTime(), DateUtil.DatePatterns.CHINA_DEFAULT));
            entity.setTitle(domain.getTitle());
            entity.setEnable(domain.getEnable());

            sysNoticeRepo.save(entity);
        }
        //修改
        else {
            entity = GlobalHelper.getEntity(sysNoticeRepo, domain.getId(), SysNoticeEntity.class);
            if (null == entity) {
                throw new StatusException("007005", "请求参数不正确请重试");
            }

            entity.setContent(domain.getContent());
            entity.setStartTime(DateUtil.parse(domain.getStartTime(), DateUtil.DatePatterns.CHINA_DEFAULT));
            entity.setEndTime(DateUtil.parse(domain.getEndTime(), DateUtil.DatePatterns.CHINA_DEFAULT));
            entity.setRootOrgId(domain.getRootOrgId() == null ? -1 : domain.getRootOrgId());
            entity.setTitle(domain.getTitle());
            entity.setEnable(domain.getEnable());

            sysNoticeRepo.save(entity);
        }

        //保存通知数据到阿里云
        SysNoticeDomain resultNotice = new SysNoticeDomain();
        resultNotice.setId(entity.getId());
        resultNotice.setRootOrgId(entity.getRootOrgId());
        resultNotice.setContent(entity.getContent());
        resultNotice.setStartTime(DateUtil.format(entity.getStartTime(), DateUtil.DatePatterns.CHINA_DEFAULT));
        resultNotice.setEndTime(DateUtil.format(entity.getEndTime(), DateUtil.DatePatterns.CHINA_DEFAULT));
        resultNotice.setTitle(entity.getTitle());
        resultNotice.setEnable(entity.getEnable());

        String prettyJson = JsonUtil.toPrettyJson(resultNotice);

        String fileSuffix = ".json";
        String filePath = systemConfig.getTempDataDir() + File.separator + "notice"
                + File.separator + resultNotice.getRootOrgId() + fileSuffix;
        filePath = PathUtil.getCanonicalPath(filePath);
        File file = new File(filePath);

        try {
            FileUtils.write(file, prettyJson, "UTF-8");

            //通用存储
            YunPathInfo pi = null;
            FileStoragePathEnvInfo env = new FileStoragePathEnvInfo();
            env.setFileSuffix(fileSuffix);
            String strRootOrgId = -1 == resultNotice.getRootOrgId()
                    ? "all"
                    : String.valueOf(resultNotice.getRootOrgId());
            env.setRootOrgId(strRootOrgId);
            pi = FileStorageUtil.saveFile("sysNotice", env, file, null);

            return pi.getUrl();//返回通知保存的地址

        } catch (IOException e) {
            throw new ExamCloudRuntimeException(e);
        } finally {
            FileUtils.deleteQuietly(file);
        }
    }
}
