package cn.com.qmth.examcloud.core.basic.api.controller;

import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.api.commons.security.enums.RoleMeta;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.service.UserDataRuleService;
import cn.com.qmth.examcloud.core.basic.service.bean.datarule.*;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.*;

@RestController
@Api(tags = "用户数据权限相关接口")
@RequestMapping("${$rmp.ctr.basic}/user/data/rule")
public class UserDataRuleController extends ControllerSupport {

    @Autowired
    private UserDataRuleService userDataRuleService;

    @PostMapping("/default/status")
    @ApiOperation(value = "获取用户数据权限的“默认全部”状态")
    public boolean defaultStatus(@RequestParam Long userId, @RequestParam DataRuleType type) {
        return userDataRuleService.getUserDataRuleForDefaultStatus(userId, type);
    }

    @PostMapping("/default/status/update")
    @ApiOperation(value = "修改用户数据权限的“默认全部”状态")
    public void updateDefaultStatus(@RequestParam Long userId, @RequestParam DataRuleType type, @RequestParam boolean enabled) {
        this.validateRoles();
        userDataRuleService.updateUserDataRuleForDefaultStatus(userId, type, enabled);
    }

    @PostMapping("/list/for/org")
    @ApiOperation(value = "获取用户“学习中心”数据权限列表（分页）")
    public Page<UserDataRuleForOrgInfo> listForOrg(@RequestBody UserDataRuleQuery req) {
        return userDataRuleService.getUserDataRuleListForOrg(req);
    }

    @PostMapping("/list/for/exam")
    @ApiOperation(value = "获取用户“考试”数据权限列表（分页）")
    public Page<UserDataRuleForExamInfo> listForExam(@RequestBody UserDataRuleQuery req) {
        return userDataRuleService.getUserDataRuleListForExam(req);
    }

    @PostMapping("/list/for/course")
    @ApiOperation(value = "获取用户“课程”数据权限列表（分页）")
    public Page<UserDataRuleForCourseInfo> listForCourse(@RequestBody UserDataRuleQuery req) {
        return userDataRuleService.getUserDataRuleListForCourse(req);
    }

    @PostMapping("/add")
    @ApiOperation(value = "新增用户数据权限")
    public void add(@RequestBody UserDataRuleForm req) {
        this.validateRoles();
        userDataRuleService.addUserDataRules(req);
    }

    @PostMapping("/delete")
    @ApiOperation(value = "删除用户数据权限")
    public void delete(@RequestBody UserDataRuleForm req) {
        this.validateRoles();
        userDataRuleService.deleteUserDataRule(req);
    }

    @PostMapping("/copy")
    @ApiOperation(value = "复制用户数据权限")
    public void copy(@RequestBody UserDataRuleCopyReq req) {
        this.validateRoles();
        userDataRuleService.copyUserDataRule(req);
    }

    @Naked
    @PostMapping("/check")
    @ApiOperation(value = "验证用户数据权限")
    public boolean checkUserDataRule(@RequestParam Long userId, @RequestParam DataRuleType type, @RequestParam Long refId) {
        return userDataRuleService.checkUserDataRule(userId, type, refId);
    }

    private void validateRoles() {
        User accessUser = getAccessUser();
        if (!hasAnyRoles(accessUser, RoleMeta.SUPER_ADMIN, RoleMeta.ORG_ADMIN)) {
            throw new StatusException("500403", "没有数据操作权限！");
        }
    }

}
