package cn.com.qmth.examcloud.core.basic.api.provider;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.*;
import cn.com.qmth.examcloud.core.basic.api.response.*;
import cn.com.qmth.examcloud.core.basic.dao.OrgPropertyRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.service.OrgService;
import cn.com.qmth.examcloud.core.basic.service.bean.OrgInfo;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.persistence.criteria.Predicate;
import java.util.*;

/**
 * {@link StatusException} 状态码范围：010XXX<br>
 *
 * @author WANGWEI
 * @date 2018年8月23日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@Api(tags = "RPC-机构相关接口")
@RequestMapping("${$rmp.cloud.basic}" + "org")
public class OrgCloudServiceProvider extends ControllerSupport implements OrgCloudService {

    private static final long serialVersionUID = -7858439296389761341L;

    @Autowired
    private OrgService orgService;

    @Autowired
    private OrgRepo orgRepo;

    @Autowired
    OrgPropertyRepo orgPropertyRepo;

    @ApiOperation(value = "保存机构")
    @PostMapping("saveOrg")
    @Transactional
    @Override
    public SaveOrgResp saveOrg(@RequestBody SaveOrgReq req) {
        OrgInfo info = new OrgInfo();
        info.setCode(req.getCode());
        info.setName(req.getName());
        info.setContacts(req.getContacts());
        info.setTelephone(req.getTelephone());
        info.setDomainName(req.getDomainName());
        info.setRemark(req.getRemark());

        OrgEntity saved = orgService.saveRootOrg(info);

        SaveOrgResp orgResp = new SaveOrgResp();
        orgResp.setId(saved.getId());
        orgResp.setName(saved.getName());
        return orgResp;
    }

    @ApiOperation(value = "获取机构")
    @PostMapping("getOrg")
    @Override
    public GetOrgResp getOrg(@RequestBody GetOrgReq req) {
        Long orgId = req.getOrgId();
        String orgCode = req.getOrgCode();
        Long rootOrgId = req.getRootOrgId();

        OrgEntity org = null;
        if (null != orgId) {
            org = GlobalHelper.getEntity(orgRepo, orgId, OrgEntity.class);
        } else if (StringUtils.isNotBlank(orgCode)) {
            if (null == rootOrgId) {
                throw new StatusException("150001", "rootOrgId is null");
            }
            org = orgRepo.findByRootIdAndCode(rootOrgId, orgCode);
        } else {
            throw new StatusException("150001", "orgId,orgCode不能都不为空");
        }

        if (null == org) {
            throw new StatusException("150001", "机构不存在");
        }

        GetOrgResp resp = new GetOrgResp();
        resp.setOrg(toOrgBean(org));
        return resp;
    }

    @ApiOperation(value = "按ID集合查询机构")
    @PostMapping("getOrgsByIdList")
    @Override
    public GetOrgsByIdListResp getOrgsByIdList(@RequestBody GetOrgsByIdListReq req) {

        List<Long> orgIdList = req.getOrgIdList();

        final List<OrgBean> orgList = Lists.newArrayList();
        for (Long cur : orgIdList) {
            OrgEntity org = GlobalHelper.getEntity(orgRepo, cur, OrgEntity.class);
            if (null == org) {
                throw new StatusException("010001", "no org. [id=" + cur + "]");
            }

            OrgBean orgBean = toOrgBean(org);
            orgList.add(orgBean);
        }
        GetOrgsByIdListResp resp = new GetOrgsByIdListResp();
        resp.setOrgList(orgList);
        return resp;
    }

    @Override
    @PostMapping("getOrgMaps")
    @ApiOperation(value = "根据ID列表获取机构集合")
    public GetOrgMapsResp getOrgMaps(@RequestBody GetOrgMapsReq req) {
        Map<Long, OrgEntity> orgs = orgService.getOrgMapsByIds(req.getOrgIds());

        Map<Long, OrgBean> orgMaps = new HashMap<>();
        for (Map.Entry<Long, OrgEntity> entry : orgs.entrySet()) {
            orgMaps.put(entry.getKey(), toOrgBean(entry.getValue()));
        }

        GetOrgMapsResp resp = new GetOrgMapsResp();
        resp.setOrgMaps(orgMaps);
        return resp;
    }

    @ApiOperation(value = "查询所有机构")
    @PostMapping("getOrgs")
    @Override
    public GetOrgsResp getOrgs(@RequestBody GetOrgsReq req) {
        Boolean withoutParentId = req.getWithoutParentId();
        Long rootOrgId = req.getRootOrgId();
        Boolean enable = req.getEnable();

        final long start = null == req.getStart() ? 1 : req.getStart();

        Pageable pageable = PageRequest.of(0, 100, Sort.Direction.ASC, "id");

        Specification<OrgEntity> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();

            if (null != rootOrgId) {
                predicates.add(cb.equal(root.get("rootId"), rootOrgId));
            }

            predicates.add(cb.greaterThanOrEqualTo(root.get("id"), start));

            if (null != withoutParentId) {
                if (withoutParentId) {
                    predicates.add(cb.isNull(root.get("parentId")));
                } else {
                    predicates.add(cb.isNotNull(root.get("parentId")));
                }
            }

            if (null != enable) {
                predicates.add(cb.equal(root.get("enable"), enable));
            }

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Page<OrgEntity> page = orgRepo.findAll(specification, pageable);

        Iterator<OrgEntity> iterator = page.iterator();

        List<OrgBean> list = Lists.newArrayList();
        long next = start;
        boolean has = false;
        while (iterator.hasNext()) {
            OrgEntity org = iterator.next();
            next = org.getId();

            OrgBean orgBean = toOrgBean(org);
            list.add(orgBean);
            has = true;
        }

        GetOrgsResp resp = new GetOrgsResp();
        if (has) {
            next++;
        }
        resp.setNext(next);
        resp.setOrgBeanList(list);

        return resp;
    }

    private OrgBean toOrgBean(OrgEntity entity) {
        OrgBean bean = new OrgBean();
        bean.setId(entity.getId());
        bean.setParentId(entity.getParentId());
        bean.setRootId(entity.getRootId());
        bean.setCode(entity.getCode());
        bean.setName(entity.getName());
        bean.setEnable(entity.getEnable());
        return bean;
    }

}
