package cn.com.qmth.examcloud.core.basic.api.provider;

import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.commons.util.DateUtil.DatePatterns;
import cn.com.qmth.examcloud.core.basic.api.SystemPropertyCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.GetSystemPropertyReq;
import cn.com.qmth.examcloud.core.basic.api.request.SetSystemPropertyReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetSystemPropertyResp;
import cn.com.qmth.examcloud.core.basic.api.response.SetSystemPropertyResp;
import cn.com.qmth.examcloud.core.basic.service.SystemPropertyService;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;

/**
 * 系统配置
 *
 * @author WANGWEI
 * @date 2018年12月3日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@Api(tags = "RPC-系统配置相关接口")
@RequestMapping("${$rmp.cloud.basic}" + "systemProperty")
public class SystemPropertyCloudServiceProvider extends ControllerSupport
        implements
        SystemPropertyCloudService {

    private static final long serialVersionUID = 2299432976949405946L;

    @Autowired
    SystemPropertyService systemPropertyService;

    @ApiOperation(value = "设置配置")
    @PostMapping("setSystemProperty")
    @Transactional
    @Override
    public SetSystemPropertyResp setSystemProperty(@RequestBody SetSystemPropertyReq req) {
        String key = req.getKey();
        String value = req.getValue();
        systemPropertyService.set(key, value);

        SetSystemPropertyResp resp = new SetSystemPropertyResp();
        resp.setKey(key);
        resp.setValue(value);
        return resp;
    }

    @ApiOperation(value = "获取配置")
    @PostMapping("getSystemProperty")
    @Override
    public GetSystemPropertyResp getSystemProperty(@RequestBody GetSystemPropertyReq req) {
        String key = req.getKey();
        Object value = systemPropertyService.get(key);

        String strValue = null;
        if (null == value) {
            strValue = null;
        } else if (value instanceof Date) {
            strValue = DateUtil.format((Date) value, DatePatterns.CHINA_DEFAULT);
        } else {
            strValue = String.valueOf(value);
        }

        GetSystemPropertyResp resp = new GetSystemPropertyResp();
        resp.setKey(key);
        resp.setValue(strValue);
        return resp;
    }

}
