package cn.com.qmth.examcloud.core.basic.api.provider;

import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.api.commons.security.enums.RoleMeta;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.ByteUtil;
import cn.com.qmth.examcloud.commons.util.SHA256;
import cn.com.qmth.examcloud.core.basic.api.UserCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.RoleBean;
import cn.com.qmth.examcloud.core.basic.api.bean.UserBean;
import cn.com.qmth.examcloud.core.basic.api.request.AddUserReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetAllUsersByRoleReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetUserListByIdsReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetUserReq;
import cn.com.qmth.examcloud.core.basic.api.response.AddUserResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetAllUsersByRoleResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetUserListByIdsResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetUserResp;
import cn.com.qmth.examcloud.core.basic.base.constants.BasicConsts;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRoleRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.RoleEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserRoleRelationEntity;
import cn.com.qmth.examcloud.core.basic.service.UserDataRuleService;
import cn.com.qmth.examcloud.core.basic.service.UserService;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * {@link StatusException} 状态码范围：008XXX<br>
 *
 * @author WANGWEI
 * @date 2018年7月23日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@Api(tags = "RPC-用户相关接口")
@RequestMapping("${$rmp.cloud.basic}" + "user")
public class UserCloudServiceProvider extends ControllerSupport implements UserCloudService {

    private static final long serialVersionUID = -7881522922704971610L;

    @Autowired
    UserRepo userRepo;

    @Autowired
    OrgRepo orgRepo;

    @Autowired
    RoleRepo roleRepo;

    @Autowired
    UserService userService;

    @Autowired
    UserRoleRelationRepo userRoleRelationRepo;

    @Autowired
    UserDataRuleService userDataRuleService;

    @ApiOperation(value = "保存用户")
    @PostMapping("addUser")
    @Transactional
    @Override
    public AddUserResp addUser(@RequestBody AddUserReq req) {
        trim(req);
        Long rootOrgId = req.getRootOrgId();
        String loginName = req.getLoginName();
        String name = req.getName();

        UserEntity userEntity = userRepo.findByRootOrgIdAndLoginName(rootOrgId, loginName);

        if (null != userEntity) {
            throw new StatusException("650001", "用户名已经存在");
        }

        userEntity = new UserEntity();
        userEntity.setEnable(true);
        userEntity.setLoginName(loginName);
        userEntity.setRootOrgId(rootOrgId);
        userEntity.setName(name);
        if (StringUtils.isBlank(req.getPassword())) {
            userEntity.setPassword(ByteUtil.toHexAscii(SHA256.encode(userEntity.getLoginName() + BasicConsts.DEFAULT_PASSWORD)));
        } else {
            userEntity.setPassword(ByteUtil.toHexAscii(SHA256.encode(userEntity.getLoginName() + req.getPassword())));
        }

        UserEntity saved = userService.save(userEntity);

        List<String> roleCodeList = req.getRoleCodeList();

        if (CollectionUtils.isEmpty(roleCodeList)) {
            throw new StatusException("650002", "角色不能为空");
        }

        List<UserRoleRelationEntity> userRoles = Lists.newArrayList();
        for (String roleCode : roleCodeList) {
            RoleEntity roleEntity = roleRepo.findByCodeAndRootOrgIdIsNull(roleCode);
            if (null == roleEntity) {
                roleEntity = roleRepo.findByCodeAndRootOrgId(roleCode, rootOrgId);
            }
            if (null == roleEntity) {
                throw new StatusException("002002", "role code is wrong. roleCode=" + roleCode);
            }
            if (roleEntity.getCode().equals(RoleMeta.SUPER_ADMIN.name())) {
                throw new StatusException("150007", "不允许新增或修改超级管理员");
            }
            UserRoleRelationEntity relation = new UserRoleRelationEntity(saved.getId(),
                    roleEntity.getId());
            userRoles.add(relation);
        }

        userRoleRelationRepo.deleteByUserId(saved.getId());
        userRoleRelationRepo.saveAll(userRoles);

        if (saved.getOrgId() != null) {
            // 用户与学习中心的数据权限
            userDataRuleService.addUserDataRule(saved.getId(), DataRuleType.ORG, saved.getOrgId());
        }

        AddUserResp resp = new AddUserResp();
        resp.setUserId(saved.getId());
        return resp;
    }

    @ApiOperation(value = "获取用户")
    @PostMapping("getUser")
    @Override
    public GetUserResp getUser(@RequestBody GetUserReq req) {
        Long rootOrgId = req.getRootOrgId();
        String loginName = req.getLoginName();
        Long userId = req.getUserId();

        UserEntity userEntity = null;
        if (null != userId) {
            userEntity = GlobalHelper.getEntity(userRepo, userId, UserEntity.class);
        } else if (StringUtils.isNotBlank(loginName)) {
            userEntity = userRepo.findByRootOrgIdAndLoginName(rootOrgId, loginName);
        } else {
            throw new StatusException("230001", "userId and  loginName can not be all null");
        }

        GetUserResp resp = new GetUserResp();
        if (null == userEntity) {
            resp.setExisting(false);
            return resp;
        }

        resp.setExisting(true);

        UserBean userBean = new UserBean();
        userBean.setUserId(userEntity.getId());
        userBean.setName(userEntity.getName());
        userBean.setLoginName(userEntity.getLoginName());
        userBean.setDisplayName(userEntity.getLoginName() + " (" + userEntity.getName() + ")");
        userBean.setOrgId(userEntity.getOrgId());
        userBean.setRootOrgId(userEntity.getRootOrgId());
        OrgEntity rootOrg = GlobalHelper.getEntity(orgRepo, userBean.getRootOrgId(),
                OrgEntity.class);
        userBean.setRootOrgName(rootOrg.getName());

        if (null != userBean.getOrgId()) {
            OrgEntity org = GlobalHelper.getEntity(orgRepo, userBean.getOrgId(), OrgEntity.class);
            userBean.setOrgName(org.getName());
        }

        List<RoleBean> roleList = getUserRoles(userEntity.getId());
        userBean.setRoleList(roleList);

        resp.setUserBean(userBean);
        return resp;
    }

    /**
     * 获取角色集合
     *
     * @param userId
     * @return
     * @author WANGWEI
     */
    private List<RoleBean> getUserRoles(Long userId) {
        List<UserRoleRelationEntity> relationList = userRoleRelationRepo.findAllByUserId(userId);
        List<RoleBean> roleList = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(relationList)) {
            for (UserRoleRelationEntity cur : relationList) {
                Long roleId = cur.getRoleId();
                RoleEntity roleEntity = GlobalHelper.getEntity(roleRepo, roleId, RoleEntity.class);
                if (null == roleEntity) {
                    throw new StatusException("002002", "roleId wrong. roleId=" + roleId);
                }
                RoleBean role = new RoleBean(roleEntity.getRootOrgId(), roleEntity.getId(),
                        roleEntity.getCode(), roleEntity.getName());
                roleList.add(role);
            }
        }
        return roleList;
    }

    @ApiOperation(value = "根据角色获取用户集合")
    @PostMapping("getAllUsersByRole")
    @Override
    public GetAllUsersByRoleResp getAllUsersByRole(@RequestBody GetAllUsersByRoleReq req) {
        Long rootOrgId = req.getRootOrgId();
        Long roleId = req.getRoleId();
        String roleCode = req.getRoleCode();

        if (null != roleId) {
            RoleEntity roleEntity = GlobalHelper.getEntity(roleRepo, roleId, RoleEntity.class);
            if (null == roleEntity) {
                throw new StatusException("150001", "角色不存在");
            }
        } else if (StringUtils.isNotBlank(roleCode)) {
            RoleEntity roleEntity = roleRepo.findByCodeAndRootOrgIdIsNull(roleCode.trim());
            if (null == roleEntity) {
                roleEntity = roleRepo.findByCodeAndRootOrgId(roleCode.trim(), rootOrgId);
            }
            if (null == roleEntity) {
                throw new StatusException("150002", "角色不存在");
            }
            roleId = roleEntity.getId();
        } else {
            throw new StatusException("150003", "roleId,roleCode不能都为空");
        }
        final Long finalRoleId = roleId;

        final long start = null == req.getStart() ? 1 : req.getStart();

        Pageable pageable = PageRequest.of(0, 100, Sort.Direction.ASC, "id");

        Specification<UserEntity> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));

            predicates.add(cb.greaterThanOrEqualTo(root.get("id"), start));

            Subquery<UserRoleRelationEntity> subquery = query
                    .subquery(UserRoleRelationEntity.class);
            Root<UserRoleRelationEntity> subRoot = subquery.from(UserRoleRelationEntity.class);
            subquery.select(subRoot.get("userId"));
            Predicate p1 = cb.equal(subRoot.get("roleId"), finalRoleId);
            Predicate p2 = cb.equal(subRoot.get("userId"), root.get("id"));
            subquery.where(cb.and(p1, p2));
            predicates.add(cb.exists(subquery));

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Page<UserEntity> page = userRepo.findAll(specification, pageable);

        Iterator<UserEntity> iterator = page.iterator();

        List<UserBean> list = Lists.newArrayList();
        long next = start;
        boolean has = false;
        while (iterator.hasNext()) {
            UserEntity userEntity = iterator.next();
            UserBean userBean = new UserBean();
            userBean.setUserId(userEntity.getId());
            userBean.setName(userEntity.getName());
            userBean.setLoginName(userEntity.getLoginName());
            userBean.setDisplayName(userEntity.getLoginName() + " (" + userEntity.getName() + ")");
            userBean.setOrgId(userEntity.getOrgId());
            userBean.setRootOrgId(userEntity.getRootOrgId());

            next = userEntity.getId();
            list.add(userBean);
            has = true;
        }

        GetAllUsersByRoleResp resp = new GetAllUsersByRoleResp();
        if (has) {
            next++;
        }
        resp.setNext(next);
        resp.setUserBeanList(list);

        return resp;
    }

    @ApiOperation(value = "根据ID获取用户集合")
    @PostMapping("getUserListByIds")
    @Override
    public GetUserListByIdsResp getUserListByIds(@RequestBody GetUserListByIdsReq req) {
        Long rootOrgId = req.getRootOrgId();
        List<Long> userIdList = req.getUserIdList();
        if (CollectionUtils.isEmpty(userIdList)) {
            throw new StatusException("008001", "userIdList is empty");
        }
        if (100 < userIdList.size()) {
            throw new StatusException("008002", "size of userIdList is too large");
        }

        List<UserEntity> userEntityList = userRepo.findByIdIn(userIdList);

        List<UserBean> list = Lists.newArrayList();
        for (UserEntity userEntity : userEntityList) {
            UserBean userBean = new UserBean();
            userBean.setUserId(userEntity.getId());
            userBean.setName(userEntity.getName());
            userBean.setLoginName(userEntity.getLoginName());
            userBean.setDisplayName(userEntity.getLoginName() + " (" + userEntity.getName() + ")");
            userBean.setOrgId(userEntity.getOrgId());
            userBean.setRootOrgId(userEntity.getRootOrgId());

            if (!userBean.getRootOrgId().equals(rootOrgId)) {
                throw new StatusException("008003", "params error");
            }
            list.add(userBean);
        }

        GetUserListByIdsResp resp = new GetUserListByIdsResp();
        resp.setUserBeanList(list);

        return resp;
    }

}
