package cn.com.qmth.examcloud.core.basic.api.provider;

import cn.com.qmth.examcloud.core.basic.api.UserDataRuleCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.AddUserDataRuleReq;
import cn.com.qmth.examcloud.core.basic.api.request.CheckUserDataRuleReq;
import cn.com.qmth.examcloud.core.basic.api.request.QueryUserDataRuleReq;
import cn.com.qmth.examcloud.core.basic.api.response.AddUserDataRuleResp;
import cn.com.qmth.examcloud.core.basic.api.response.CheckUserDataRuleResp;
import cn.com.qmth.examcloud.core.basic.api.response.QueryUserDataRuleResp;
import cn.com.qmth.examcloud.core.basic.service.UserDataRuleService;
import cn.com.qmth.examcloud.core.basic.service.bean.datarule.UserDataRuleForm;
import cn.com.qmth.examcloud.core.basic.service.cache.UserDataRuleCacheBean;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Api(tags = "RPC-用户数据权限相关接口")
@RequestMapping("${$rmp.cloud.basic}/user/data/rule")
public class UserDataRuleCloudServiceProvider extends ControllerSupport implements UserDataRuleCloudService {

    private static final long serialVersionUID = 1L;

    @Autowired
    private UserDataRuleService userDataRuleService;

    @Override
    @PostMapping("/query")
    @ApiOperation(value = "_查询用户的数据权限")
    public QueryUserDataRuleResp queryUserDataRule(@RequestBody QueryUserDataRuleReq req) {
        UserDataRuleCacheBean cacheBean = userDataRuleService.loadUserDataRuleCache(req.getUserId(), req.getType());

        QueryUserDataRuleResp resp = new QueryUserDataRuleResp();
        resp.setGlobalStatus(cacheBean.getGlobalStatus());
        resp.setRefIds(cacheBean.getRefIds());
        return resp;
    }

    @Override
    @PostMapping("/check")
    @ApiOperation(value = "_验证用户的数据权限")
    public CheckUserDataRuleResp checkUserDataRule(@RequestBody CheckUserDataRuleReq req) {
        boolean pass = userDataRuleService.checkUserDataRule(req.getUserId(), req.getType(), req.getRefId());

        CheckUserDataRuleResp resp = new CheckUserDataRuleResp();
        resp.setPass(pass);
        return resp;
    }

    @Override
    @PostMapping("/add")
    @ApiOperation(value = "_新增用户数据权限")
    public AddUserDataRuleResp addUserDataRule(@RequestBody AddUserDataRuleReq req) {
        UserDataRuleForm form = new UserDataRuleForm();
        form.setUserId(req.getUserId());
        form.setType(req.getType());
        form.setRefIds(req.getRefIds());

        userDataRuleService.addUserDataRules(form);
        return new AddUserDataRuleResp();
    }

}
