package cn.com.qmth.examcloud.core.basic.service;

import cn.com.qmth.examcloud.api.commons.enums.DataRuleType;
import cn.com.qmth.examcloud.core.basic.service.bean.datarule.*;
import cn.com.qmth.examcloud.core.basic.service.cache.UserDataRuleCacheBean;
import org.springframework.data.domain.Page;

public interface UserDataRuleService {

    /**
     * 获取用户数据权限的“默认全部”状态
     */
    boolean getUserDataRuleForDefaultStatus(Long userId, DataRuleType type);

    /**
     * 修改用户数据权限的“默认全部”状态
     */
    void updateUserDataRuleForDefaultStatus(Long userId, DataRuleType type, boolean enabled);

    /**
     * 获取用户数据权限列表（分页）
     */
    Page<UserDataRuleInfo> getUserDataRuleList(UserDataRuleQuery req);

    /**
     * 获取用户“学习中心”数据权限列表（分页）
     */
    Page<UserDataRuleForOrgInfo> getUserDataRuleListForOrg(UserDataRuleQuery req);

    /**
     * 获取用户“考试”数据权限列表（分页）
     */
    Page<UserDataRuleForExamInfo> getUserDataRuleListForExam(UserDataRuleQuery req);

    /**
     * 获取用户“课程”数据权限列表（分页）
     */
    Page<UserDataRuleForCourseInfo> getUserDataRuleListForCourse(UserDataRuleQuery req);

    /**
     * 新增用户数据权限
     */
    void addUserDataRules(UserDataRuleForm req);

    /**
     * 新增用户数据权限
     */
    void addUserDataRule(Long userId, DataRuleType type, Long refId);

    /**
     * 删除用户数据权限
     */
    void deleteUserDataRule(UserDataRuleForm req);

    /**
     * 复制用户数据权限
     */
    void copyUserDataRule(UserDataRuleCopyReq req);

    /**
     * 验证用户的数据权限
     */
    boolean checkUserDataRule(Long userId, DataRuleType type, Long refId);

    /**
     * 获取用户的数据权限（缓存）
     */
    UserDataRuleCacheBean loadUserDataRuleCache(Long userId, DataRuleType type);

}
