package cn.com.qmth.examcloud.core.basic.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.dao.AdminOperateRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRoleRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.AdminOperateEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.RoleEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserRoleRelationEntity;
import cn.com.qmth.examcloud.core.basic.service.AdminOperateService;
import cn.com.qmth.examcloud.core.basic.service.bean.AdminOperateBean;
import cn.com.qmth.examcloud.core.basic.service.bean.AdminOperateQuery;
import cn.com.qmth.examcloud.reports.commons.bean.AdminOperateReport;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;

@Service
public class AdminOperateServiceImpl implements AdminOperateService {

	@Autowired
	private JdbcTemplate jdbcTemplate;
	@Autowired
	UserRepo userRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	UserRoleRelationRepo userRoleRelationRepo;
	@Autowired
	AdminOperateRepo adminOperateRepo;

	@Override
	public void saveOperate(AdminOperateReport r, String msgId) {
		AdminOperateEntity e = new AdminOperateEntity();
		e.setMsgId(msgId);
		e.setRootOrgId(r.getRootOrgId());
		e.setOperateUserId(r.getOperateUserId());
		e.setOperateTime(r.getReportTime());
		e.setOperateIp(r.getRemoteHost());
		e.setOperate(r.getOperate());
		e.setOperateInfo(r.getOperateInfo());
		e.setCreationTime(new Date());
		try {
			adminOperateRepo.save(e);
		} catch (DuplicateKeyException e1) {
			// 忽略;
		}
	}

	@Override
	public PageInfo<AdminOperateBean> queryPage(AdminOperateQuery req, Integer pageNo, Integer pageSize,
			Long rootOrgId) {
		Long count = jdbcTemplate.queryForObject(" select count(1) from (" + getSql(req, rootOrgId) + ") tem", Long.class);
		List<AdminOperateBean> list = new ArrayList<AdminOperateBean>();
		if (count > 0) {
			RowMapper<AdminOperateBean> rowMapper = new BeanPropertyRowMapper<AdminOperateBean>(AdminOperateBean.class);
			list = jdbcTemplate.query(
					getSql(req, rootOrgId) + " order by t.id desc limit " + ((pageNo - 1) * pageSize) + "," + pageSize,
					rowMapper);
			for (AdminOperateBean bean:list) {
				List<UserRoleRelationEntity> relationList = userRoleRelationRepo.findAllByUserId(bean.getOperateUserId());
				List<String> roleNameList = Lists.newArrayList();
				for (UserRoleRelationEntity cur : relationList) {
					RoleEntity curRoleEntity = GlobalHelper.getEntity(roleRepo, cur.getRoleId(), RoleEntity.class);
					if (null == curRoleEntity) {
						throw new StatusException("150002", "角色错误");
					}
					roleNameList.add(curRoleEntity.getName());
				}
				bean.setRoles(StringUtils.join(roleNameList, ","));
			}
		}
		PageInfo<AdminOperateBean> ret = new PageInfo<AdminOperateBean>();
		ret.setList(list);
		ret.setTotal(count);
		return ret;
	}

	private String getSql(AdminOperateQuery req, Long rootOrgId) {
		String loginName = req.getLoginName();
		String startTime = req.getStartTime();
		String endTime = req.getEndTime();
		Long roleId = req.getRoleId();

		StringBuilder sb = new StringBuilder();
		sb.append(
				" select t.operate_time operateTime,t.operate_ip operateIp,t.operate_user_id operateUserId ");
		sb.append(" ,t.operate,t.operate_info operateInfo,f.login_name loginName,f.name from ec_b_admin_operate t ");
		sb.append(" inner join ec_b_user f on t.operate_user_id=f.id ");
		sb.append(" where 1=1  ");
		if (rootOrgId != null) {
			sb.append(" and t.root_org_id=").append(rootOrgId);
		}
		if (StringUtils.isNotBlank(loginName)) {
			sb.append(" and f.login_name like '").append(loginName).append("%'  ");
		}
		if (StringUtils.isNotBlank(endTime)) {
			sb.append(" and t.operate_time <=STR_TO_DATE('").append(endTime).append("','%Y-%m-%d %H:%i:%s') ");
		}
		if (StringUtils.isNotBlank(startTime)) {
			sb.append(" and t.operate_time >=STR_TO_DATE('").append(startTime).append("','%Y-%m-%d %H:%i:%s') ");
		}
		if (roleId != null) {
			sb.append(" and EXISTS (select 1 from ec_b_user_role_relation h where h.role_id=").append(roleId)
					.append(" and h.user_id=t.operate_user_id)");
		}
		return sb.toString();
	}

}
