package cn.com.qmth.examcloud.core.basic.service.impl;

import java.util.List;
import java.util.Optional;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.PrivilegeGroupRepo;
import cn.com.qmth.examcloud.core.basic.dao.PrivilegeRepo;
import cn.com.qmth.examcloud.core.basic.dao.RolePrivilegeRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.RootOrgPrivilegeRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRoleRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.PrivilegeEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.PrivilegeGroupEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.RoleEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.RolePrivilegeRelationEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.RolePrivilegeRelationPK;
import cn.com.qmth.examcloud.core.basic.dao.entity.RootOrgPrivilegeRelationEntity;
import cn.com.qmth.examcloud.core.basic.service.RolePrivilegeService;
import cn.com.qmth.examcloud.core.basic.service.bean.RoleInfo;
import cn.com.qmth.examcloud.core.basic.service.cache.PrrivilegeRolesCache;
import cn.com.qmth.examcloud.core.basic.service.cache.RootOrgPrrivilegesCache;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年6月14日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class RolePrivilegeServiceImpl implements RolePrivilegeService {

	@Autowired
	PrivilegeRepo privilegeRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	RolePrivilegeRelationRepo rolePrivilegeRelationRepo;

	@Autowired
	RootOrgPrivilegeRelationRepo rootOrgPrivilegeRelationRepo;

	@Autowired
	PrivilegeGroupRepo privilegeGroupRepo;

	@Autowired
	UserRoleRelationRepo userRoleRelationRepo;

	@Autowired
	PrrivilegeRolesCache prrivilegeRolesCache;

	@Autowired
	RootOrgPrrivilegesCache rootOrgPrrivilegesCache;

	@Override
	public void updateRolePrivilegeRelations(Long rootOrgId, Long roleId, Long privilegeGroupId,
			Set<Long> privilegeIdSet) {

		RoleEntity roleEntity = GlobalHelper.getEntity(roleRepo, roleId, RoleEntity.class);
		Long roleRootOrgId = roleEntity.getRootOrgId();
		if (null != roleRootOrgId) {
			if (!roleRootOrgId.equals(rootOrgId)) {
				throw new StatusException("620150", "角色和机构不匹配");
			}
		}

		GlobalHelper.getPresentEntity(privilegeGroupRepo, privilegeGroupId,
				PrivilegeGroupEntity.class);

		for (Long cur : privilegeIdSet) {
			PrivilegeEntity privilegeEntity = GlobalHelper.getPresentEntity(privilegeRepo, cur,
					PrivilegeEntity.class);
			if (!privilegeEntity.getGroupId().equals(privilegeGroupId)) {
				throw new StatusException("620151", "权限和权限组不匹配");
			}
		}

		List<PrivilegeEntity> privilegeList = privilegeRepo
				.findAllByGroupIdOrderByWeightDesc(privilegeGroupId);

		for (PrivilegeEntity cur : privilegeList) {
			RolePrivilegeRelationPK pk = new RolePrivilegeRelationPK(roleId, cur.getId(),
					rootOrgId);
			Optional<RolePrivilegeRelationEntity> optional = rolePrivilegeRelationRepo.findById(pk);
			if (optional.isPresent()) {
				rolePrivilegeRelationRepo.delete(optional.get());
			}
		}

		for (Long cur : privilegeIdSet) {
			RolePrivilegeRelationEntity bean = new RolePrivilegeRelationEntity();
			bean.setRoleId(roleId);
			bean.setRootOrgId(rootOrgId);
			bean.setPrivilegeId(cur);

			rolePrivilegeRelationRepo.saveAndFlush(bean);
		}

		for (Long cur : privilegeIdSet) {
			PrivilegeEntity privilegeEntity = GlobalHelper.getPresentEntity(privilegeRepo, cur,
					PrivilegeEntity.class);
			prrivilegeRolesCache.remove(rootOrgId, privilegeEntity.getCode());
		}

	}

	@Override
	public void updateRootOrgPrivilegeRelations(Long rootOrgId, Long privilegeGroupId,
			Set<Long> privilegeIdSet) {

		GlobalHelper.getPresentEntity(privilegeGroupRepo, privilegeGroupId,
				PrivilegeGroupEntity.class);

		rootOrgPrivilegeRelationRepo.deleteByRootOrgIdAndGroupId(rootOrgId, privilegeGroupId);

		for (Long cur : privilegeIdSet) {
			if(cur==-1){
				continue;
			}
			PrivilegeEntity privilegeEntity = GlobalHelper.getPresentEntity(privilegeRepo, cur,
					PrivilegeEntity.class);
			if (!privilegeEntity.getGroupId().equals(privilegeGroupId)) {
				throw new StatusException("620158", "权限和权限组不匹配");
			}

			RootOrgPrivilegeRelationEntity bean = new RootOrgPrivilegeRelationEntity();
			bean.setRootOrgId(rootOrgId);
			bean.setPrivilegeId(cur);
			bean.setGroupId(privilegeGroupId);

			rootOrgPrivilegeRelationRepo.saveAndFlush(bean);
		}

		rootOrgPrrivilegesCache.remove(rootOrgId);
	}

	@Override
	public RoleEntity saveRole(RoleInfo info) {
		String code = info.getCode();
		String name = info.getName();
		Long rootOrgId = info.getRootOrgId();

		if (StringUtils.isBlank(code)) {
			throw new StatusException("620001", "code is blank");
		}

		if (StringUtils.isBlank(name)) {
			throw new StatusException("620002", "name is blank");
		}

		if (null == rootOrgId) {
			throw new StatusException("620003", "rootOrgId is null");
		}

		RoleEntity roleEntity = roleRepo.findByCodeAndRootOrgId(code, rootOrgId);

		if (null == roleEntity) {
			roleEntity = new RoleEntity();
			roleEntity.setCode(code);
			roleEntity.setName(name);
			roleEntity.setRootOrgId(rootOrgId);
		}

		RoleEntity saved = roleRepo.save(roleEntity);

		return saved;
	}

	@Override
	public RoleEntity deleteRole(RoleInfo info, boolean cascade) {
		Long id = info.getId();
		String code = info.getCode();
		Long rootOrgId = info.getRootOrgId();

		if (null == rootOrgId) {
			throw new StatusException("620001", "name is blank");
		}

		RoleEntity roleEntity = null;
		if (null != id) {
			roleEntity = GlobalHelper.getEntity(roleRepo, id, RoleEntity.class);
		} else if (StringUtils.isNotBlank(code)) {
			roleEntity = roleRepo.findByCodeAndRootOrgId(code, rootOrgId);
		} else {
			throw new StatusException("620002", "id,code can not be all null");
		}

		if (null == roleEntity) {
			throw new StatusException("620003", "roleEntity is null");
		}

		if (!cascade) {
			if (0 < rolePrivilegeRelationRepo.countByRoleIdAndRootOrgId(roleEntity.getId(),
					rootOrgId)) {
				throw new StatusException("620004", "角色已关联权限");
			}

			if (0 < userRoleRelationRepo.countByRoleId(roleEntity.getId())) {
				throw new StatusException("620004", "角色已关联用户");
			}
		}

		roleRepo.delete(roleEntity);

		rolePrivilegeRelationRepo.deleteByRoleIdAndRootOrgId(roleEntity.getId(), rootOrgId);
		userRoleRelationRepo.deleteByRoleId(roleEntity.getId());

		return roleEntity;
	}

}
