package cn.com.qmth.examcloud.core.basic.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.api.commons.enums.BasicDataType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.commons.util.DateUtil.DatePatterns;
import cn.com.qmth.examcloud.core.basic.dao.SystemPropertyRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.SystemPropertyEntity;
import cn.com.qmth.examcloud.core.basic.service.SystemPropertyService;
import cn.com.qmth.examcloud.core.basic.service.bean.SystemPropertyInfo;
import cn.com.qmth.examcloud.core.basic.service.cache.SystemPropertyCache;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;

/**
 * 系统配置服务
 *
 * @author WANGWEI
 * @date 2018年12月3日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class SystemPropertyServiceImpl implements SystemPropertyService {

	@Autowired
	SystemPropertyRepo systemPropertyRepo;

	@Autowired
	SystemPropertyCache systemPropertyCache;

	@Override
	public void set(String key, String value) {
		if (StringUtils.isBlank(key)) {
			throw new StatusException("350001", "key is blank");
		}

		SystemPropertyEntity sysConf = GlobalHelper.getEntity(systemPropertyRepo, key,
				SystemPropertyEntity.class);
		if (null == sysConf) {
			throw new StatusException("350002", "key is wrong");
		}
		BasicDataType dataType = sysConf.getPropValueType();

		if (StringUtils.isBlank(value)) {
			sysConf.setPropValue(null);
		} else if (dataType.equals(BasicDataType.STRING)) {
			sysConf.setPropValue(value);
		} else if (dataType.equals(BasicDataType.LONG)) {
			try {
				sysConf.setPropValue(String.valueOf(Long.parseLong(value)));
			} catch (NumberFormatException e) {
				throw new StatusException("350002", "value is wrong");
			}
		} else if (dataType.equals(BasicDataType.INTEGER)) {
			try {
				sysConf.setPropValue(String.valueOf(Integer.parseInt(value)));
			} catch (NumberFormatException e) {
				throw new StatusException("350002", "value is wrong");
			}
		} else if (dataType.equals(BasicDataType.DATE)) {
			try {
				sysConf.setPropValue(
						DateUtil.format(DateUtil.parse(value, DatePatterns.CHINA_DEFAULT),
								DatePatterns.CHINA_DEFAULT));
			} catch (Exception e) {
				throw new StatusException("350002", "value is wrong");
			}
		} else if (dataType.equals(BasicDataType.BOOLEAN)) {
			if ("true".equals(value)) {
				sysConf.setPropValue(value);
			} else if ("false".equals(value)) {
				sysConf.setPropValue(value);
			} else {
				throw new StatusException("350002", "value is wrong");
			}
		}

	}

	@Override
	public Object get(String key) {
		SysPropertyCacheBean sysPropertyCacheBean = systemPropertyCache.get(key);
		if (null == sysPropertyCacheBean) {
			return null;
		}
		return sysPropertyCacheBean.getValue();
	}

	@Override
	public void addSysProp(SystemPropertyInfo info) {

		SystemPropertyEntity sysPropEntity = GlobalHelper.getEntity(systemPropertyRepo,
				info.getPropKey(), SystemPropertyEntity.class);
		if (null != sysPropEntity) {
			throw new StatusException("70001", "参数名称已存在");
		}
		sysPropEntity = new SystemPropertyEntity();
		sysPropEntity.setPropKey(info.getPropKey());
		sysPropEntity.setPropValueType(info.getPropValueType());
		sysPropEntity.setPropValue(null);
		sysPropEntity.setDescription(info.getDescription());

		SystemPropertyEntity saved = systemPropertyRepo.save(sysPropEntity);
		set(info.getPropKey(), info.getPropValue());
		systemPropertyCache.refresh(saved.getPropKey());
	}

	@Override
	public void updateSysProp(SystemPropertyInfo info) {

		SystemPropertyEntity sysPropEntity = GlobalHelper.getEntity(systemPropertyRepo,
				info.getPropKey(), SystemPropertyEntity.class);
		if (null == sysPropEntity) {
			throw new StatusException("70001", "参数名称不存在");
		}
		sysPropEntity.setPropValueType(info.getPropValueType());
		sysPropEntity.setPropValue(null);
		sysPropEntity.setDescription(info.getDescription());

		SystemPropertyEntity saved = systemPropertyRepo.save(sysPropEntity);
		set(info.getPropKey(), info.getPropValue());
		systemPropertyCache.refresh(saved.getPropKey());
	}

}
